USER_NAME={{ env.user_name }}
USER_PASSWORD={{ env.user_passwd }}
GID_NUMBER={{ env.user_gid }}
UID_NUMBER={{ env.user_uid }}
SSH_PORT={{ env.ssh_port }}
DOCKER_IMAGE_ROOT={{ env.base_docker_image }}
CONAN_LOGIN_USERNAME="{{ env.conan_username }}"
CONAN_PASSWORD="{{ env.conan_passwd }}"