###
# @copyright Copyright (C) 2022 Spyrosoft Solutions. All rights reserved.
# THIS FILE WAS GENERATED AUTOMATICALLY. DO NOT CHANGE IT.
###
version: '3.9'

services:
  {{ config.project.name }}_dev:
    image: {{ config.project.docker_image_tag }}
    platform: linux/amd64
    build:
      context: .
      # in case of missing .env file (eg. CI/CD) system will use default values
      args:
        USER_NAME: ${USER_NAME:-user}
        USER_PASSWORD: ${USER_PASSWORD:-user}
        GID_NUMBER: ${GID_NUMBER:-1000}
        UID_NUMBER: ${UID_NUMBER:-1000}
        DOCKER_IMAGE_ROOT: ${DOCKER_IMAGE_ROOT:-ubuntu:20.04}
        CONAN_LOGIN_USERNAME: ${CONAN_LOGIN_USERNAME:-""}
        CONAN_PASSWORD: ${CONAN_PASSWORD:-""}
    privileged: true
    volumes:
      - ..:/workspace
      - /dev:/dev
    command: sleep infinity
  {% if config.project.in_repo_conan_cache %}
    environment:
      - CONAN_USER_HOME=/workspace
  {% endif %}
  {% if config.docker_compose.ports %}
    ports:
    {% for port_ranges in config.docker_compose.ports %}
      - "{{ port_ranges }}"
    {% endfor %}
  {% endif %}
