/**
 * @copyright Copyright (C) 2022 Spyrosoft Solutions. All rights reserved.
 */

#include <gtest/gtest.h>
#include <memory>
{% for inc in header.includes %}
#include "{{inc.name}}"
{% endfor %}
#include "{{header.name}}"

{% if header.namespaces %}
{% for namespace in header.namespaces %}
using namespace {{namespace.name}};
{% endfor %}
{% endif %}
using namespace ::testing;

{% if header.classes %}
{% for cls in header.classes %}
class {{cls.name}}Test : public Test
{
public:
    void SetUp() override
    {
    }

    void TearDown() override
    {
    }
};
{% endfor %}
{% endif %}

{% if header.classes %}
{% for cls in header.classes %}
TEST_F({{cls.name}}Test, exampleTest)
{
}
{% endfor %}
{% endif %}