{% if config.project.bin_name %}
add_executable({{ config.project.bin_name|lower }})
target_sources({{ config.project.bin_name|lower }} PRIVATE ${CMAKE_CURRENT_SOURCE_DIR}/{{ config.project.bin_name|lower }}.cpp)

{% endif %}
{% if config.project.lib_name %}
add_library(${PROJECT_NAME} STATIC
    ${CMAKE_CURRENT_SOURCE_DIR}/{{ config.project.lib_name|lower }}.cpp
)

target_include_directories(${PROJECT_NAME}
    PUBLIC
        $<BUILD_INTERFACE:${CMAKE_SOURCE_DIR}>/{{ config.include_dir_path.relative_to(config.project_root) }}
    PRIVATE
        $<BUILD_INTERFACE:${CMAKE_SOURCE_DIR}>
)

{% endif %}
{% if config.project.lib_name %}
install(TARGETS ${PROJECT_NAME} DESTINATION lib)
install(DIRECTORY ${CMAKE_SOURCE_DIR}/{{ config.include_dir_path.relative_to(config.project_root) }}/ DESTINATION {{ config.include_dir_path.relative_to(config.project_root) }})
{% endif %}
